@@ -1,4 +1,6 @@
 import asyncio
+
+from asyncio import Task
 from datetime import datetime
 from functools import partial
 from typing import List
@@ -16,11 +18,13 @@ from .api.http_client import HttpClient
 from .domain.adapters_initializer import AdaptersInitializer
 from .domain.collector_config import CollectorConfig
 from .domain.collector_config_loader import CollectorConfigLoader
+from .logger import logger
+
 
 
 class Collector:
     def __init__(
-        self, config_path: str, root_package: str, plugin_factory: PluginFactory
+            self, config_path: str, root_package: str, plugin_factory: PluginFactory
     ) -> None:
         loader = CollectorConfigLoader(config_path, plugin_factory)
         self.config: CollectorConfig = loader.load()
@@ -58,6 +62,14 @@ class Collector:
         async with ClientSession() as session:
             return await self.__api.register_datasource(request, session)
 
+    async def __do_tasks(self, tasks: List[Task]):
+        finished, not_finished = await asyncio.wait(tasks, return_when=asyncio.FIRST_EXCEPTION)
+
+        for task in finished:
+            if task.exception():
+                logger.error("Error", exc_info=task.exception())
+                await self.__do_tasks(not_finished)
+
     async def __ingest_data(self):
         async with ClientSession() as session:
             send_request = partial(self.__send_request, session=session)
@@ -65,8 +77,7 @@ class Collector:
                 asyncio.create_task(send_request(adapter=adapter))
                 for adapter, _ in self.adapters_with_plugins
             ]
-
-            await asyncio.gather(*tasks)
+            await self.__do_tasks(tasks)
 
     async def __get_data_entity_list(self, adapter: AbstractAdapter):
         result = adapter.get_data_entity_list()
